@@ -59,7 +59,7 @@ int poptype(char *mailboxname)
    return 0;
 }
 
-int imap(int sd, int *nm, int *om, char *user, char *password)
+int imap(int sd, int *nm, int *om, char *user, char *password, char *mailbox)
 {
    char input[200], output[200], strg[10];
    char *pnt;
@@ -75,6 +75,7 @@ int imap(int sd, int *nm, int *om, char *user, char *password)
    Swrite(sd, output);
    if (Sread(sd, input, 200, TIMEOUT) <= 0)
       return -1;
+   while (input[0] == '*') if (Sread(sd, input, 200, TIMEOUT) <= 0) return -1;
 
    sscanf(input, "%8s", strg);
    if (strncmp(input, "A000 OK", 7)) {
@@ -85,7 +86,7 @@ int imap(int sd, int *nm, int *om, char *user, char *password)
       return -1;
    }
    
-   sprintf(output, "A001 SELECT INBOX\n");
+   sprintf(output, "A001 SELECT %s\n", mailbox);
    Swrite(sd, output);
 
    do {
@@ -100,8 +101,10 @@ int imap(int sd, int *nm, int *om, char *user, char *password)
    Swrite(sd, output);
    if (Sread(sd, input, 200, TIMEOUT) <= 0)
       return -1;
+   while (input[0] == '*') if (Sread(sd, input, 200, TIMEOUT) <= 0) return -1;
+
    if (strncmp(input, "A002 OK", 7)) {
-      if (DEBUG) fprintf(stderr, "That jerk doesn't know CHECK!\n");
+      if (DEBUG) fprintf(stderr, "That moron doesn't know CHECK!\n");
       sprintf(output, "A003 LOGOUT\n");
       Swrite(sd, output);
       return -1;
@@ -122,12 +125,14 @@ int imap(int sd, int *nm, int *om, char *user, char *password)
    
    if (Sread(sd, input, 200, TIMEOUT) <= 0)
       return -1;
+   while (input[0] == '*') if (Sread(sd, input, 200, TIMEOUT) <= 0) return -1;
    if (strncmp(input, "A003 OK", 7))
-      if (DEBUG) fprintf(stderr, "WARNING: SEARCH UNSEEN did not terminate with OK\n");
+      if (DEBUG) fprintf(stderr, "WARNING: SEARCH UNSEEN did not terminate with OK. Ignoring..\n");
 
    sprintf(output, "A004 LOGOUT\n");
    Swrite(sd, output);
    Sread(sd, input, 200, TIMEOUT);
+   while (input[0] == '*') Sread(sd, input, 200, TIMEOUT);
    return 0;
 }
 
@@ -168,7 +173,10 @@ int pop3(int sd, int *nm, int *om, char *user, char *password)
     /* LIST works */
         pnt = input + strlen(strg) + 1;
         i = sscanf(pnt, "%d", om);
-    } else *om = 0;
+    } else {
+       *om = 0;
+       if (DEBUG) fprintf(stdout, "Your POP server does not support the LAST command.\n");
+    }
     
     /* We've got it. Now let's go. */
     Swrite(sd, "QUIT\n");
@@ -178,7 +186,7 @@ int pop3(int sd, int *nm, int *om, char *user, char *password)
 
 int pop(char *string, int *nmail, int *oldmail)
 /* String must contain - in this order:
-    servername username password [port#] 
+    servername username password [port#] [mailbox (for IMAP only)]
     
     nmail: total number of available mail
     oldmail: number of old mail			*/
@@ -186,7 +194,7 @@ int pop(char *string, int *nmail, int *oldmail)
 {
     SOCKET *sock;
     int sd, i, port = 0, type;
-    char server[100], user[100], password[100];
+    char server[100], user[100], password[100], mailbox[100];
     char *pnt;
     static int InDuty = 0;
 
@@ -197,24 +205,29 @@ int pop(char *string, int *nmail, int *oldmail)
         return -1;
     InDuty = 1;
 
+    /* Parse the string */
+    /* Servername */
     pnt = string + 2;
     if (sscanf(pnt, "%100s", server) != 1) {
         InDuty = 0;
         return -1;
     }
     
+    /* Username */
     pnt += strlen(server) + 1;
     if (sscanf(pnt, "%100s", user) != 1) {
         InDuty = 0;
         return -1;
     }
     
+    /* Password */
     pnt += strlen(user) + 1;
     if (sscanf(pnt, "%100s", password) != 1) {
         InDuty = 0;
         return -1;
     }
     
+    /* optional portnumber */
     pnt += strlen(password) + 1;
     if (sscanf(pnt, "%d", &port) != 1)
        switch (type) {
@@ -222,6 +235,17 @@ int pop(char *string, int *nmail, int *oldmail)
           case TYPE_IMAP: port = DEFAULTIMAPPORT; break;
        }
 
+    /* optional mailboxname for IMAP */
+    if (type == TYPE_IMAP) {
+       if (sscanf(pnt, "%100s", &mailbox) != 1)
+          strcpy(mailbox, "INBOX");
+       else {
+          int i;
+          for (i=0; mailbox[i] != '\0'; i++)
+             mailbox[i] = toupper(mailbox[i]);
+       }
+    }
+    
     /* Create socket */
     if (!(sock = Sopen())) {
         InDuty = 0;
@@ -242,7 +266,7 @@ int pop(char *string, int *nmail, int *oldmail)
                         return -1;
                       }
                       break;
-       case TYPE_IMAP: if (imap(sd, nmail, oldmail, user, password) == -1) {
+       case TYPE_IMAP: if (imap(sd, nmail, oldmail, user, password, mailbox) == -1) {
           		if (DEBUG) fprintf(stderr, "IMAP connection failed\n");
                         InDuty = 0;
                         return -1;
